@@ -50,6 +50,7 @@
 #include "xsettings-manager.h"
 #include "fontconfig-monitor.h"
 #include "gsd-remote-display-manager.h"
+#include "wm-button-layout-translation.h"
 
 #define GNOME_XSETTINGS_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GNOME_TYPE_XSETTINGS_MANAGER, GnomeXSettingsManagerPrivate))
 
@@ -58,6 +59,8 @@
 #define INTERFACE_SETTINGS_SCHEMA "org.gnome.desktop.interface"
 #define SOUND_SETTINGS_SCHEMA     "org.gnome.desktop.sound"
 #define PRIVACY_SETTINGS_SCHEMA     "org.gnome.desktop.privacy"
+#define WM_SETTINGS_SCHEMA        "org.gnome.desktop.wm.preferences"
+#define CLASSIC_WM_SETTINGS_SCHEMA "org.gnome.shell.extensions.classic-overrides"
 
 #define XSETTINGS_PLUGIN_SCHEMA "org.gnome.settings-daemon.plugins.xsettings"
 #define XSETTINGS_OVERRIDE_KEY  "overrides"
@@ -262,7 +265,7 @@ struct _FixedEntry {
 struct GnomeXSettingsManagerPrivate
 {
         guint              start_idle_id;
-        XSettingsManager **managers;
+        XSettingsManager  *manager;
         GHashTable        *settings;
 
         GSettings         *plugin_settings;
@@ -305,12 +308,8 @@ translate_bool_int (GnomeXSettingsManager *manager,
                     TranslationEntry      *trans,
                     GVariant              *value)
 {
-        int i;
-
-        for (i = 0; manager->priv->managers [i]; i++) {
-                xsettings_manager_set_int (manager->priv->managers [i], trans->xsetting_name,
-                                           g_variant_get_boolean (value));
-        }
+        xsettings_manager_set_int (manager->priv->manager, trans->xsetting_name,
+                                   g_variant_get_boolean (value));
 }
 
 static void
@@ -318,12 +317,8 @@ translate_int_int (GnomeXSettingsManager *manager,
                    TranslationEntry      *trans,
                    GVariant              *value)
 {
-        int i;
-
-        for (i = 0; manager->priv->managers [i]; i++) {
-                xsettings_manager_set_int (manager->priv->managers [i], trans->xsetting_name,
-                                           g_variant_get_int32 (value));
-        }
+        xsettings_manager_set_int (manager->priv->manager, trans->xsetting_name,
+                                   g_variant_get_int32 (value));
 }
 
 static void
@@ -331,35 +326,58 @@ translate_string_string (GnomeXSettingsManager *manager,
                          TranslationEntry      *trans,
                          GVariant              *value)
 {
-        int i;
+        xsettings_manager_set_string (manager->priv->manager,
+                                      trans->xsetting_name,
+                                      g_variant_get_string (value, NULL));
+}
 
-        for (i = 0; manager->priv->managers [i]; i++) {
-                xsettings_manager_set_string (manager->priv->managers [i],
-                                              trans->xsetting_name,
-                                              g_variant_get_string (value, NULL));
+static void
+translate_button_layout (GnomeXSettingsManager *manager,
+                         TranslationEntry      *trans,
+                         GVariant              *value)
+{
+        GSettings *classic_settings;
+        GVariant *classic_value = NULL;
+        const char *session;
+        char *layout;
+
+        /* Hack: until we get session-dependent defaults in GSettings,
+         *       swap out the usual schema for the "classic" one when
+         *       running in classic mode
+         */
+        session = g_getenv ("XDG_CURRENT_DESKTOP");
+        classic_settings = g_hash_table_lookup (manager->priv->settings,
+                                                CLASSIC_WM_SETTINGS_SCHEMA);
+        if (session && strstr (session, "GNOME-Classic") && classic_settings) {
+                classic_value = g_settings_get_value (classic_settings, "button-layout");
+                layout = g_variant_dup_string (classic_value, NULL);
+        } else {
+                layout = g_variant_dup_string (value, NULL);
         }
+
+        translate_wm_button_layout_to_gtk (layout);
+
+        xsettings_manager_set_string (manager->priv->manager,
+                                      trans->xsetting_name,
+                                      layout);
+
+        if (classic_value)
+                g_variant_unref (classic_value);
+        g_free (layout);
 }
 
 static void
 fixed_false_int (GnomeXSettingsManager *manager,
                  FixedEntry            *fixed)
 {
-        int i;
-
-        for (i = 0; manager->priv->managers [i]; i++) {
-                xsettings_manager_set_int (manager->priv->managers [i], fixed->xsetting_name, FALSE);
-        }
+        xsettings_manager_set_int (manager->priv->manager, fixed->xsetting_name, FALSE);
 }
 
 static void
 fixed_true_int (GnomeXSettingsManager *manager,
                 FixedEntry            *fixed)
 {
-        int i;
-
-        for (i = 0; manager->priv->managers [i]; i++) {
-                xsettings_manager_set_int (manager->priv->managers [i], fixed->xsetting_name, TRUE);
-        }
+        xsettings_manager_set_int (manager->priv->manager, fixed->xsetting_name, TRUE);
 }
 
 static FixedEntry fixed_entries [] = {
@@ -404,17 +422,15 @@ static TranslationEntry translations [] = {
         { "org.gnome.desktop.sound", "input-feedback-sounds",      "Net/EnableInputFeedbackSounds", translate_bool_int },
 
         { "org.gnome.desktop.privacy", "recent-files-max-age",      "Gtk/RecentFilesMaxAge", translate_int_int },
-        { "org.gnome.desktop.privacy", "remember-recent-files",    "Gtk/RecentFilesEnabled", translate_bool_int }
+        { "org.gnome.desktop.privacy", "remember-recent-files",    "Gtk/RecentFilesEnabled", translate_bool_int },
+        { "org.gnome.desktop.wm.preferences", "button-layout",     "Gtk/DecorationLayout", translate_button_layout }
 };
 
 static gboolean
 notify_idle (gpointer data)
 {
         GnomeXSettingsManager *manager = data;
-        gint i;
-        for (i = 0; manager->priv->managers [i]; i++) {
-                xsettings_manager_notify (manager->priv->managers[i]);
-        }
+        xsettings_manager_notify (manager->priv->manager);
         manager->priv->notify_idle_id = 0;
         return G_SOURCE_REMOVE;
 }
@@ -426,6 +442,7 @@ queue_notify (GnomeXSettingsManager *manager)
                 return;
 
         manager->priv->notify_idle_id = g_idle_add (notify_idle, manager);
+        g_source_set_name_by_id (manager->priv->notify_idle_id, "[gnome-settings-daemon] notify_idle");
 }
 
 static double
@@ -692,20 +709,17 @@ static void
 xft_settings_set_xsettings (GnomeXSettingsManager *manager,
                             GnomeXftSettings      *settings)
 {
-        int i;
-
         gnome_settings_profile_start (NULL);
 
-        for (i = 0; manager->priv->managers [i]; i++) {
-                xsettings_manager_set_int (manager->priv->managers [i], "Xft/Antialias", settings->antialias);
-                xsettings_manager_set_int (manager->priv->managers [i], "Xft/Hinting", settings->hinting);
-                xsettings_manager_set_string (manager->priv->managers [i], "Xft/HintStyle", settings->hintstyle);
-                xsettings_manager_set_int (manager->priv->managers [i], "Gdk/WindowScalingFactor", settings->window_scale);
-                xsettings_manager_set_int (manager->priv->managers [i], "Gdk/UnscaledDPI", settings->dpi);
-                xsettings_manager_set_int (manager->priv->managers [i], "Xft/DPI", settings->scaled_dpi);
-                xsettings_manager_set_string (manager->priv->managers [i], "Xft/RGBA", settings->rgba);
-                xsettings_manager_set_int (manager->priv->managers [i], "Gtk/CursorThemeSize", settings->cursor_size);
-        }
+        xsettings_manager_set_int (manager->priv->manager, "Xft/Antialias", settings->antialias);
+        xsettings_manager_set_int (manager->priv->manager, "Xft/Hinting", settings->hinting);
+        xsettings_manager_set_string (manager->priv->manager, "Xft/HintStyle", settings->hintstyle);
+        xsettings_manager_set_int (manager->priv->manager, "Gdk/WindowScalingFactor", settings->window_scale);
+        xsettings_manager_set_int (manager->priv->manager, "Gdk/UnscaledDPI", settings->dpi);
+        xsettings_manager_set_int (manager->priv->manager, "Xft/DPI", settings->scaled_dpi);
+        xsettings_manager_set_string (manager->priv->manager, "Xft/RGBA", settings->rgba);
+        xsettings_manager_set_int (manager->priv->manager, "Gtk/CursorThemeSize", settings->cursor_size);
+
         gnome_settings_profile_end (NULL);
 }
 
@@ -811,13 +825,10 @@ override_callback (GSettings             *settings,
                    GnomeXSettingsManager *manager)
 {
         GVariant *value;
-        int i;
 
         value = g_settings_get_value (settings, XSETTINGS_OVERRIDE_KEY);
 
-        for (i = 0; manager->priv->managers[i]; i++) {
-                xsettings_manager_set_overrides (manager->priv->managers[i], value);
-        }
+        xsettings_manager_set_overrides (manager->priv->manager, value);
         queue_notify (manager);
 
         g_variant_unref (value);
@@ -844,19 +855,14 @@ gtk_modules_callback (GsdXSettingsGtk       *gtk,
                       GnomeXSettingsManager *manager)
 {
         const char *modules = gsd_xsettings_gtk_get_modules (manager->priv->gtk);
-        int i;
 
         if (modules == NULL) {
-                for (i = 0; manager->priv->managers [i]; ++i) {
-                        xsettings_manager_delete_setting (manager->priv->managers [i], "Gtk/Modules");
-                }
+                xsettings_manager_delete_setting (manager->priv->manager, "Gtk/Modules");
         } else {
                 g_debug ("Setting GTK modules '%s'", modules);
-                for (i = 0; manager->priv->managers [i]; ++i) {
-                        xsettings_manager_set_string (manager->priv->managers [i],
-                                                      "Gtk/Modules",
-                                                      modules);
-                }
+                xsettings_manager_set_string (manager->priv->manager,
+                                              "Gtk/Modules",
+                                              modules);
         }
 
         queue_notify (manager);
@@ -866,14 +872,11 @@ static void
 fontconfig_callback (fontconfig_monitor_handle_t *handle,
                      GnomeXSettingsManager       *manager)
 {
-        int i;
         int timestamp = time (NULL);
 
         gnome_settings_profile_start (NULL);
 
-        for (i = 0; manager->priv->managers [i]; i++) {
-                xsettings_manager_set_int (manager->priv->managers [i], "Fontconfig/Timestamp", timestamp);
-        }
+        xsettings_manager_set_int (manager->priv->manager, "Fontconfig/Timestamp", timestamp);
         queue_notify (manager);
         gnome_settings_profile_end (NULL);
 }
@@ -900,6 +903,7 @@ start_fontconfig_monitor (GnomeXSettingsManager  *manager)
         fontconfig_cache_init ();
 
         manager->priv->start_idle_id = g_idle_add ((GSourceFunc) start_fontconfig_monitor_idle_cb, manager);
+        g_source_set_name_by_id (manager->priv->start_idle_id, "[gnome-settings-daemon] start_fontconfig_monitor_idle_cb");
 
         gnome_settings_profile_end (NULL);
 }
@@ -917,15 +921,11 @@ static void
 notify_have_shell (GnomeXSettingsManager   *manager,
                    gboolean                 have_shell)
 {
-        int i;
-
         gnome_settings_profile_start (NULL);
         if (manager->priv->have_shell == have_shell)
                 return;
         manager->priv->have_shell = have_shell;
-        for (i = 0; manager->priv->managers [i]; i++) {
-                xsettings_manager_set_int (manager->priv->managers [i], "Gtk/ShellShowsAppMenu", have_shell);
-        }
+        xsettings_manager_set_int (manager->priv->manager, "Gtk/ShellShowsAppMenu", have_shell);
         queue_notify (manager);
         gnome_settings_profile_end (NULL);
 }
@@ -963,6 +963,11 @@ find_translation_entry (GSettings *settings, const char *key)
 
         g_object_get (settings, "schema", &schema, NULL);
 
+        if (g_str_equal (schema, CLASSIC_WM_SETTINGS_SCHEMA)) {
+              g_free (schema);
+              schema = g_strdup (WM_SETTINGS_SCHEMA);
+        }
+
         for (i = 0; i < G_N_ELEMENTS (translations); i++) {
                 if (g_str_equal (schema, translations[i].gsettings_schema) &&
                     g_str_equal (key, translations[i].gsettings_key)) {
@@ -982,7 +987,6 @@ xsettings_callback (GSettings             *settings,
                     GnomeXSettingsManager *manager)
 {
         TranslationEntry *trans;
-        guint             i;
         GVariant         *value;
 
         if (g_str_equal (key, TEXT_SCALING_FACTOR_KEY) ||
@@ -1002,11 +1006,9 @@ xsettings_callback (GSettings             *settings,
 
         g_variant_unref (value);
 
-        for (i = 0; manager->priv->managers [i]; i++) {
-                xsettings_manager_set_string (manager->priv->managers [i],
-                                              "Net/FallbackIconTheme",
-                                              "gnome");
-        }
+        xsettings_manager_set_string (manager->priv->manager,
+                                      "Net/FallbackIconTheme",
+                                      "gnome");
         queue_notify (manager);
 }
 
@@ -1028,13 +1030,10 @@ static gboolean
 setup_xsettings_managers (GnomeXSettingsManager *manager)
 {
         GdkDisplay *display;
-        int         i;
-        int         n_screens;
         gboolean    res;
         gboolean    terminated;
 
         display = gdk_display_get_default ();
-        n_screens = gdk_display_get_n_screens (display);
 
         res = xsettings_manager_check_running (gdk_x11_display_get_xdisplay (display),
                                                gdk_screen_get_number (gdk_screen_get_default ()));
@@ -1044,22 +1043,14 @@ setup_xsettings_managers (GnomeXSettingsManager *manager)
                 return FALSE;
         }
 
-        manager->priv->managers = g_new0 (XSettingsManager *, n_screens + 1);
-
         terminated = FALSE;
-        for (i = 0; i < n_screens; i++) {
-                GdkScreen *screen;
-
-                screen = gdk_display_get_screen (display, i);
-
-                manager->priv->managers [i] = xsettings_manager_new (gdk_x11_display_get_xdisplay (display),
-                                                                     gdk_screen_get_number (screen),
-                                                                     terminate_cb,
-                                                                     &terminated);
-                if (! manager->priv->managers [i]) {
-                        g_warning ("Could not create xsettings manager for screen %d!", i);
-                        return FALSE;
-                }
+        manager->priv->manager = xsettings_manager_new (gdk_x11_display_get_xdisplay (display),
+                                                        gdk_screen_get_number (gdk_screen_get_default ()),
+                                                        terminate_cb,
+                                                        &terminated);
+        if (! manager->priv->manager) {
+                g_warning ("Could not create xsettings manager!");
+                return FALSE;
         }
 
         return TRUE;
@@ -1085,7 +1076,6 @@ force_disable_animation_changed (GObject    *gobject,
                                  GnomeXSettingsManager *manager)
 {
         gboolean force_disable, value;
-        int i;
 
         g_object_get (gobject, "force-disable-animations", &force_disable, NULL);
         if (force_disable)
@@ -1097,9 +1087,17 @@ force_disable_animation_changed (GObject    *gobject,
                 value = g_settings_get_boolean (settings, "enable-animations");
         }
 
-        for (i = 0; manager->priv->managers [i]; i++) {
-                xsettings_manager_set_int (manager->priv->managers [i], "Gtk/EnableAnimations", value);
-        }
+        xsettings_manager_set_int (manager->priv->manager, "Gtk/EnableAnimations", value);
+
+        queue_notify (manager);
+}
+
+static void
+enable_animations_changed_cb (GSettings             *settings,
+                              gchar                 *key,
+                              GnomeXSettingsManager *manager)
+{
+        force_disable_animation_changed (G_OBJECT (manager->priv->remote_display), NULL, manager);
 }
 
 static void
@@ -1133,6 +1131,7 @@ gboolean
 gnome_xsettings_manager_start (GnomeXSettingsManager *manager,
                                GError               **error)
 {
+        GSettingsSchema *schema;
         GVariant    *overrides;
         guint        i;
         GList       *list, *l;
@@ -1168,6 +1167,20 @@ gnome_xsettings_manager_start (GnomeXSettingsManager *manager,
                              SOUND_SETTINGS_SCHEMA, g_settings_new (SOUND_SETTINGS_SCHEMA));
         g_hash_table_insert (manager->priv->settings,
                              PRIVACY_SETTINGS_SCHEMA, g_settings_new (PRIVACY_SETTINGS_SCHEMA));
+        g_hash_table_insert (manager->priv->settings,
+                             WM_SETTINGS_SCHEMA, g_settings_new (WM_SETTINGS_SCHEMA));
+
+        schema = g_settings_schema_source_lookup (g_settings_schema_source_get_default (),
+                                                  CLASSIC_WM_SETTINGS_SCHEMA, FALSE);
+        if (schema) {
+                g_hash_table_insert (manager->priv->settings,
+                                     CLASSIC_WM_SETTINGS_SCHEMA,
+                                     g_settings_new_full (schema, NULL, NULL));
+                g_settings_schema_unref (schema);
+        }
+
+        g_signal_connect (G_OBJECT (g_hash_table_lookup (manager->priv->settings, INTERFACE_SETTINGS_SCHEMA)), "changed::enable-animations",
+                          G_CALLBACK (enable_animations_changed_cb), manager);
 
         for (i = 0; i < G_N_ELEMENTS (fixed_entries); i++) {
                 FixedEntry *fixed = &fixed_entries[i];
@@ -1216,15 +1229,12 @@ gnome_xsettings_manager_start (GnomeXSettingsManager *manager,
 
         start_shell_monitor (manager);
 
-        for (i = 0; manager->priv->managers [i]; i++)
-                xsettings_manager_set_string (manager->priv->managers [i],
-                                              "Net/FallbackIconTheme",
-                                              "gnome");
+        xsettings_manager_set_string (manager->priv->manager,
+                                      "Net/FallbackIconTheme",
+                                      "gnome");
 
         overrides = g_settings_get_value (manager->priv->plugin_settings, XSETTINGS_OVERRIDE_KEY);
-        for (i = 0; manager->priv->managers [i]; i++) {
-                xsettings_manager_set_overrides (manager->priv->managers [i], overrides);
-        }
+        xsettings_manager_set_overrides (manager->priv->manager, overrides);
         queue_notify (manager);
         g_variant_unref (overrides);
 
@@ -1238,7 +1248,6 @@ void
 gnome_xsettings_manager_stop (GnomeXSettingsManager *manager)
 {
         GnomeXSettingsManagerPrivate *p = manager->priv;
-        int i;
 
         g_debug ("Stopping xsettings manager");
 
@@ -1254,12 +1263,9 @@ gnome_xsettings_manager_stop (GnomeXSettingsManager *manager)
         if (p->shell_name_watch_id > 0)
                 g_bus_unwatch_name (p->shell_name_watch_id);
 
-        if (p->managers != NULL) {
-                for (i = 0; p->managers [i]; ++i)
-                        xsettings_manager_destroy (p->managers [i]);
-
-                g_free (p->managers);
-                p->managers = NULL;
+        if (p->manager != NULL) {
+                xsettings_manager_destroy (p->manager);
+                p->manager = NULL;
         }
 
         if (p->plugin_settings != NULL) {
